# -*- coding: utf-8 -*-
###########################################################################
#
# Eole NG
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr
#
###########################################################################

"""generation de code cheetah sur des valeurs creole multivaluees"""

import re, sys
from creole.cfgparser import EoleDict
from creole import config
from era.backend.template import Template

def eole_dict():
    """recuperation du dictionnaire creole
    afin de recuperer l'info sur les groupe et
    les variables maitre-esclaves
    """
    if sys.platform == 'win32':
        dico = EoleDict(templatedir=config.templatedir, logfile='nul')
    else:
        dico = EoleDict(templatedir=config.templatedir, logfile='/dev/null')
    dico.read_dir(config.eoledirs)
    return dico

creole_dico = None

def get_groups():
    """ dictionnaire des groupes creole
    {maitre:[esclave1, esclave2]]}
    """
    return creole_dico.groups

def get_master(varname):
    var = creole_dico.get_variable(varname)
    return var.slave

def is_multi(varname):
    """permet de savoir si une variable creole est multiple ou non
    """
    try:
        #it could be a container variable
        var = creole_dico.get_variable(varname)
        return var.multi
    except KeyError:
        return False

class Multivar:
    """generation de code cheetah sur des valeurs multivaluees"""
    def __init__(self, lines=[], creole_dico=None):
        """:lines: lignes iptables generees par le compilateur"""
        self.lines = lines
        if creole_dico == None:
            globals()['creole_dico'] = eole_dict()
        else:
            globals()['creole_dico'] = creole_dico
        self.slaves = []
        for slaves in globals()['creole_dico'].groups.values():
            self.slaves.extend(slaves)

    def _getcreolevar_inline(self, line):
        """:return: la liste des variables creole dans une ligne
        """
        # regexp de motif %%mavariable[] ou mavariable[1]
        monovarregexp = re.compile('%%(\w*)\[[0-9]*\]')
        # regexp de motif %%mavariable
        multivarregexp = re.compile('%%(\w*)')

        # toute les variables templates sont parsees
        multivars = re.findall(multivarregexp, line)
        for var in multivars:
            # cette variable ne doit pas etre suivie de crochets
            if var not in re.findall(monovarregexp, line):
                yield var

    def _validate_group(self, master, var):
        """
        verifie que les variables presentes sont toutes dans le mme groupe
        celui de la variable (eventuelle) maitre
        :master: variable maitre
        :var: variables de la ligne
        """
        if not master:
            raise Exception("Il n'y a pas de master pour la variable slave %s" % (var))
        if not get_groups().has_key(master):
            raise Exception, "la variable %s a instancier n'est pas dans le dictionnaire creole"% (master)

        if var not in get_groups()[master]:
            raise Exception, "la variable creole multiple %s n'est pas dans le groupe de la variable maitre %s" % \
                  (var, master)

    def process(self):
        """collecte les valeurs multivaluees dans le dictionaire creole
        """
        for line in self.lines:
            masters = {}
            tpline = line
            #utilisation d'un set pour n'obtenir qu'une occurence de chaque variable
            inline_vars = set(self._getcreolevar_inline(tpline))
            for var in inline_vars:
                if var in self.slaves:
                    master = get_master(var)
                    self._validate_group(master, var)
                    #plusieurs esclaves de la même master ou master déjà rencontrée
                    masters.setdefault(master, []).append(var)
                    tpline = tpline.replace("%%" + var, "%("+ var + ")s")
                elif is_multi(var):
                    #si c'est une master ou une multi isolée
                    #ou une "container_var"
                    #considère que c'est une master
                    masters.setdefault(var, [])
                    tpline = tpline.replace("%%" + var, "%("+ var + ")s")
                else:
                    # doubles % pour les variables non multi-valuees
                    tpline = tpline.replace("%%" + var, "%%%%" + var)

            if masters != {}:
                # generation du template cheetah
                tpl = Template(masters=masters, template=tpline)
                # pour qu'il n'y ait qu'une seule ligne a chaque fois -> split
                newline = str(tpl).split('\n')
                for n in newline:
                    yield n + '\n'
            else:
                yield line
