#!/bin/bash
. /usr/share/eole/FonctionsEoleNg
. /usr/bin/ParseDico

MDIR="/usr/share/era/modeles"
EOLE_DIR="/var/lib/eole"
REGLES_CSV="/var/lib/eole/config/regles.csv"
IPSET_TMP_DIR="/var/lib/eole/ipsets"

## existance du fichier xml firewall
if [ ! -e $MDIR/${type_amon}.xml ]
then
    EchoRouge "!! Modèle ${type_amon}.xml non trouvé !!"
    exit 1
fi

# script de lancement du compilateur de regle (backend) era
export PYTHONPATH="/usr/share/:/usr/share/eole/:$PYTHONPATH"
cd /usr/share/era/

## verifions en tout premier lieu que le firewall est bien prevu pour l'authentification
if [ "$activer_nufw" != "oui"  ]
then
    if [ "`./backend/is_authenticated $MDIR/${type_amon}.xml`" = "True"  ]
    then
        EchoRouge "!! ATTENTION: le modele Era est authentifiant alors que le pare-feu n'est pas defini comme etant authentifiant !!"
        exit 1
    fi
fi

#support des regles eole-firewall
/usr/share/eole/bastion/bin/bastion
if [ $? -eq 1 ]; then
    EchoRouge "!! Erreur a la generation des regles eole-firewall !!"
    exit 1
fi

mkdir -p $IPSET_TMP_DIR
rm -f $IPSET_TMP_DIR/*

./backend/compiler -o ${EOLE_DIR}/firewall.tmpl -a /usr/share/era/backend/data/active_tags -d ${REGLES_CSV} -t ${EOLE_DIR}/regles.tmpl -s $IPSET_TMP_DIR $MDIR/${type_amon}.xml
if [ $? -eq 1 ]; then
    EchoRouge "!! Erreur lors de la generation des regles iptables par era !!"
    exit 1
fi

# compilation du fichier de règles optionnelles si nécessaire
grep -q "%" ${EOLE_DIR}/regles.tmpl
if [ $? -eq 0 ];then
    /usr/share/creole/creolecat.py -o ${REGLES_CSV} -i /etc/eole/config.eol ${EOLE_DIR}/regles.tmpl
else
    /bin/cp -f ${EOLE_DIR}/regles.tmpl ${REGLES_CSV}
fi
## authentification nufw
# instanciation "manuelle" du script de firewall
/usr/share/creole/creolecat.py  -o /sbin/lance.firewall -i /etc/eole/config.eol ${EOLE_DIR}/firewall.tmpl
# attribution des droits au fichier lance.firewall
chmod 700 /sbin/lance.firewall

## **ajout** au hosts.allow
# la **creation** du hosts.allow est sous la responsabilite de eole-firewall
#/usr/share/era/backend/hostallow.py

## horaires de nuit du pare-feu
/usr/share/era/backend/horaires_firewall -i /var/lib/eole/config/horaires.txt >> /sbin/lance.firewall

# Instanciation manuelle du modele Era :
tmp_xml=$(mktemp)
/usr/share/creole/creolecat.py  -o $tmp_xml -i /etc/eole/config.eol /usr/share/era/modeles/"$type_amon".xml
if [ ! "$?" = "0" ]; then
    EchoRouge "Impossible de générer le modèle XML Era '$type_amon'"
    exit 1
fi

## gestion de la qos
./backend/qos_era_externe $tmp_xml

# ipsets rules that comes from exceptions
mkdir -p /usr/share/era/ipsets/
rm -f /usr/share/era/ipsets/*
for ipsetfile in `ls $IPSET_TMP_DIR/*.sh 2> /dev/null`; do
    filename=`basename $ipsetfile`
    grep -q '%' $ipsetfile
    if [ $? -eq 0 ];then
        /usr/share/creole/creolecat.py $ipsetfile -o /usr/share/era/ipsets/$filename
    else
        cp -f $ipsetfile /usr/share/era/ipsets/$filename
    fi
    chmod +x /usr/share/era/ipsets/$filename
done

# lancement du script iptables
/sbin/lance.firewall #2>&1 | head -n 2

# script de generation d'ACL NuFW :
if [ "$activer_nufw" = "oui" ]; then
    if [ ! -f /etc/nufw/acls.nufw ]; then
      # si pas de fichier la première fois, on le crée
      # pour comparaison avec le fichier généré
      touch /etc/nufw/acls.nufw
    fi
    cp /etc/nufw/acls.nufw /etc/nufw/acls.nufw.bak
    ./backend/nufw_acl_plaintext -o /etc/nufw/acls.nufw $tmp_xml
    [ ! $? = 0 ] && EchoRouge "Probleme pendant la generation des directives nufw" && rm $tmp_xml && exit 1
    pgrep nuauth > /dev/null
    if [ ! $? = 0 ]; then
        /etc/init.d/nuauth start
    else
        cmp /etc/nufw/acls.nufw /etc/nufw/acls.nufw.bak >/dev/null 2>&1 || /etc/init.d/nuauth restart
    fi
fi
rm $tmp_xml

# ajout des règles pour les postes clients interdits
python /usr/share/era/postes_interdits.py

