#!/usr/bin/env python
# -*- coding: utf-8 -*-
from xml.etree import ElementTree as ET
from xml.etree.ElementTree import tostring
"""
algorithme d'héritage des modèles

#val merge : fwtree -> fwtree -> xmlstr
#val read_from_file : fname -> fwtree
#val fromstring : xmlstr -> fwtree
#val tostring: fwtree -> xmlstr
#val main: filename -> xmlstr
#val fw_recurse: fwtree -> xmlstr

def main(fname):
    fwtree = read_from_file(fname)
    return fw_recurse(fwtree)

def fw_recurse(fwtree):
    if fwtree.attrib.has_key('model'):
        fname = fwtree.attrib['model']
        fwtree2 = read_from_file(fname)
        xmlstr = merge(fwtree, fwtree2)
        fwtree3 = fromstring(xmlstr)
        return fw_recurse(fwtree3)
    else:
        return tostring(fwtree)

fname1 = "test_heritage.xml"
main(fname1)

"""

def add(flux1, flux2):
    """accumulte flux2 sur flux1 sans test d'unicite

    flux1 = [d1, d2]
    flux2 = [d'1, d'2]
    d'1 becomes d3
    d'2 becomes d4
    add(flux1, flux2) = [d1, d2, d3, d4]

    """
    # MONTANTES
    montantes = flux1.find('montantes')
    len_montantes1 = len(flux1.findall('montantes/directive'))
    montantes2 = flux2.findall('montantes/directive')
    sorted_montantes2 = sorted(montantes2, key=lambda directive: int(directive.get('priority')))
    for directive in sorted_montantes2:
        priority = str(len_montantes1 + sorted_montantes2.index(directive) + 1)
        directive.set('priority', priority)
        montantes.append(directive)
    # DESCENDANTES
    descendantes = flux1.find('descendantes')
    len_descendantes1 = len(flux1.findall('descendantes/directive'))
    descendantes2 = flux2.findall('descendantes/directive')
    sorted_descendantes2 = sorted(descendantes2, key=lambda directive: int(directive.get('priority')))
    for directive in sorted_descendantes2:
        priority = str(len_descendantes1 + sorted_descendantes2.index(directive) + 1)
        directive.set('priority', priority)
        descendantes.append(directive)

def complementary_flux(flux1, flux2):
    """returns: flux list. Les flux qui sont dans flux2 mais pas dans flux1
    """
    existing_flux = []
    complementary = []

    for flx in flux1:
        existing_flux.append( (flx.get('zoneA'), flx.get('zoneB')) )

    for flx in flux2:
        zoneA = flx.get('zoneA')
        zoneB = flx.get('zoneB')
        if (zoneA, zoneB) not in existing_flux:
            complementary.append(flx)

    return complementary

def main(fname1):
    """point d'entree
    """
    fwtree = read_from_file(fname1)
    xmlstr = fw_recurse(fwtree)
    # puts a generated name for convenience
    root = fromstring(xmlstr)
    root.attrib['name'] = "Concatenated_Do_Not_Edit"
    return tostring(root).replace('&apos;', "'").replace('&#233;', 'é').replace('&#232;', 'è')
def fw_recurse(fwtree):
    """enclenche la récursivité verticale (le fichier cible a un attribut model)
    """
    if fwtree.attrib.has_key('model'):
        fname = fwtree.attrib['model']
        if "," in fname:
            fnames = fname.split(",")
            fnames = [fname.strip() for fname in fnames]
            fwtree6 = fwtree
            for name in fnames:
                fwtree4 = read_from_file(name)
                xmlstr = merge(fwtree6, fwtree4)
                fwtree5 = fromstring(xmlstr)
                fwtree6 = fromstring(fw_recurse(fwtree5))
            return tostring(fwtree6)

        else:
            fwtree2 = read_from_file(fname)
            xmlstr = merge(fwtree, fwtree2)
            fwtree3 = fromstring(xmlstr)
            return fw_recurse(fwtree3)
    else:
        return tostring(fwtree)

def merge(fw1, fw2):
    """concatenation des nodes element tree
    """
    root = ET.Element("firewall")
    root.text = '\n    '
    # FIXME : c'est le **fw2** qui dirige les options
    root.attrib.update(fw2.attrib)
    # ____________________________________________________________
    # ajout des zones
    zones = fw1.findall("zones/zone")
    for zone in fw2.findall("zones/zone"):
        if zone.get('name') not in [z.get('name') for z in zones]:
            zones.append(zone)

    zone_root = ET.SubElement(root, "zones")
    zone_root.text = "\n        "
    for zone in zones:
        zone_root.append(zone)
    # ____________________________________________________________
    # add services
    service_root = ET.SubElement(root, "services")
    service_root.text = "\n        "
    for serv in fw1.findall('services/service'):
        service_root.append(serv)

    for serv in fw2.findall('services/service'):
        if serv.get('name') not in [servi.get('name') for servi in service_root]:
           service_root.append(serv)
    # ____________________________________________________________
    # service group
    for serv in fw1.findall('services/groupe'):
        service_root.append(serv)


    for serv in fw2.findall('services/groupe'):
        if serv.get('id') not in [servi.get('id') for servi in service_root]:
            service_root.append(serv)

    # ____________________________________________________________
    # extremites
    extremite_root = ET.SubElement(root, "extremites")
    extremite_root.text = "\n        "
    for serv in fw1.findall('extremites/extremite'):
        extremite_root.append(serv)
    for serv in fw2.findall('extremites/extremite'):
        extremite_root.append(serv)
    # ____________________________________________________________
    # ranges
    ranges_root = ET.SubElement(root, "ranges")
    ranges_root.text = "\n        "
    for serv in fw1.findall('ranges/range'):
        ranges_root.append(serv)
    for serv in fw2.findall('ranges/range'):
        ranges_root.append(serv)

    # ____________________________________________________________
    # user_group
    user_root = ET.SubElement(root, "user_groups")
    user_root.text = "\n        "
    for serv in fw1.findall('user_groups/user_group'):
        user_root.append(serv)
    for serv in fw2.findall('user_groups/user_group'):
        user_root.append(serv)

    # ____________________________________________________________
    # flux
    flux_root = ET.SubElement(root, "flux-list")
    flux_root.text = "\n        "

    flux1 = fw1.findall('flux-list/flux')
    flux2 = fw2.findall('flux-list/flux')
    different_flux =  complementary_flux(flux1, flux2)
    # accumulation des flux identiques
    for fx1 in flux1:
        for fx2 in flux2:
            if fx1.get('zoneA') == fx2.get('zoneA') and fx1.get('zoneB') == fx2.get('zoneB'):
                add(fx1, fx2)

    for flx in flux1:
        flux_root.append(flx)

    # ajout des flux complementaires
    for flx in different_flux:
        flux_root.append(flx)

    #pi = ET.ProcessingInstruction("xml", text='version="1.0" encoding="UTF-8"')
    #tree = ET.ElementTree(root)
    #tree.write("output.xml")
    # ____________________________________________________________
    # output
    # remplace les apostrophes-entities par des apostrophes simples car c'est pas du "vrai" xml
    return '<?xml version="1.0" encoding="UTF-8" ?>\n' + tostring(root, encoding='utf-8')

def read_from_file(fname):
    xml = ET.parse(fname)
    firewall = xml.getroot()
    return firewall

def fromstring(xmlstr):
    firewall = ET.fromstring(xmlstr)
    return firewall

if __name__ == '__main__':
    import sys
    fname1 = sys.argv[1]
    print main(fname1)

